/*
 * Interrupt handling.
 *
 * Copyright (C) 2015, 2016 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef CONFIG_CPU_JZ4730_MINIPC
#include "minipc.h"
#else
#include "nanonote.h"
#endif

#include "board-specific.h"
#include "board.h"
#include "lcd.h"
#include "jzlcd.h"
#include "cpu.h"
#include "irq.h"

extern vidinfo_t panel_info;

void next_pixel(unsigned short *x, unsigned short *y)
{
	(*x)++;
	if (*x >= panel_info.vl_col) {
		*x = 0;
		(*y)++;
		if (*y >= panel_info.vl_row)
			*y = 0;
	}
}

/* Tasks. */

void plot_pattern(unsigned short pixel_type, unsigned short x, unsigned short y)
{
	while (1) {
		if (pixel_type)
			test_pixel(x, y);
		else
			clear_pixel(x, y);
		next_pixel(&x, &y);
		udelay(100);
	}
}

/* Initialisation and handling. */

void irq_init()
{
	timer_init_irq();
	handle_error_level();
	init_interrupts();
	enable_interrupts();
}

void irq_handle()
{
	unsigned short i;

	/* Check interrupt identity. */

	if (REG_INTC_IPR & (1 << TIMER_CHAN_IRQ)) {

		/* Update the pixel type. */

		/* pixel_type = __gpio_get_pin(GPIO_POWER); */

		/* Clear interrupt status. */

		__intc_ack_irq(TIMER_CHAN_IRQ);
		__tcu_clear_full_match_flag(TIMER_CHAN);

	/* Handle other interrupts, anyway. */

	} else {
		for (i = 0; i < 32; i++) {
			if (REG_INTC_IPR & (1 << i))
				__intc_ack_irq(i);
		}
	}
}

void start_task()
{
	/* enter_user_mode(); */
	plot_pattern(1, 0, 0);
}

void tlb_handle()
{
	u32 context, virtual, physical;

	/* Obtain the bad virtual address. */

	asm volatile(
		"mfc0 %0, $4\n"		/* CP0_CONTEXT */
		: "=r" (context)
		); 
                        
	/* Obtain a virtual address region with 8KB resolution. */
                        
	virtual = (context & 0x007ffff0) << 9;
                
	/* The appropriate physical address depends on the current task. */

	physical = virtual;

	/*
	Request a physical region mapping two 4KB pages.
	Pages employ C=3, dirty, valid, with the task number as the ASID.
	*/
        
	map_page(virtual, physical, 4 * 1024, 0x1f, 0);
}       
